/*
 * $Id: UserAspect.aj,v 1.2 2005/07/22 11:56:09 boehm Exp $
 * (c)reated on 10.10.2004 by oliver
 */
package jrequest.base;

import java.util.*;
import org.aspectj.lang.annotation.*;

/**
 * @author oliver
 * @since 10.10.2004
 * @version $Revision: 1.2 $
 */
public aspect UserAspect {
    
    /////   Get-Pointcut   /////

    pointcut getName() :
        get(private String User.name);

    after() : getName() {
        System.out.println("name was accessed");
    }

    /////   Set-Pointcut   /////
    
    pointcut setName():
        set(private String User.name);

    after() : setName() {
        System.out.println("name was changed");
    }

    ////   this / Zusaetzliche Kriterien   /////

    pointcut callToString1() :
        call(public String toString());

    pointcut callToString2() :
        call(public String toString()) && this(jrequest.test.UserTest);

    pointcut callToString3() :
        call(public String toString()) 
            && (this(jrequest.test.UserTest)
                || this(jrequest.test.RequestTest));

    pointcut callToString4() :
        call(public String toString()) && this(junit.framework.TestCase);

    /////   this / Endlosschleife   /////
    
    pointcut getNameOf(User user):
        get(private String User.name) && this(user)
            &&!withincode(public String User.getName());

    after(User user) : getNameOf(user) {
        System.out.println("access to name " + user.getName());
    }
    
    /////   target / Begrenzung des Gueltigkeitsbereich   /////
    
    pointcut callToString5():
        call(public String toString())
             && target(jrequest.base.User);

    /////   target / Zugriff auf das Zielobjekt   /////
    
    pointcut callSetName1():
        call(public void User.setName(..));

    before() : callSetName1() {
        System.out.println("call from " + thisJoinPoint.getThis());
    }
    
    pointcut callSetName2(User user):
        call(public void User.setName(..)) && target(user);

    before(User user) : callSetName2(user) {
        System.out.println("call from " + thisJoinPoint.getThis());
        System.out.println("old name: " + user.getName());
    }

    after(User user) : callSetName2(user) {
        System.out.println("new name: " + user.getName());
    }
    
    /////   args / Zugriff auf einen Parameter   /////
    
    pointcut callSetBirth1(Date date) :
        call(public void User.setBirth(Date)) && args(date);

    after(Date date) : callSetBirth1(date) {
        System.out.println("1: date of birth set to " + date);
    }

    pointcut callSetBirth2(Object date) :
        call(public void User.setBirth(..)) && args(date);

    after(Object date) : callSetBirth2(date) {
        System.out.println("2: date of birth set to " + date);
    }
    
    /////   args / Zugriff auf mehrere Parameter   /////
    
    pointcut callSetBirth3(int year, int month, int day) :
        call(public void User.setBirth(int, int, int))
            && args(year, month, day);

    after(int year, int month, int day) : callSetBirth3(year, month, day) {
        System.out.println("3: user is born in " + year);
        System.out.println("3: birthday: " + day + "." + month + ".");
    }
    
    /////   args / leerer Pointcut   /////
    
    pointcut callSetBirth4(int year, int month, int day) :
        call(public void User.setBirth(Date)) && (args(year, month, day));

    /////   args / Zugriff auf mehrere Parameter   /////

    pointcut callSetBirth5(int year) :
        call(public void User.setBirth(..)) && args(year, int, int);

    after(int year) : callSetBirth5(year) {
        System.out.println("5: user is born in " + year);
    }

    pointcut callSetBirth6(Object date) :
        call(public void User.setBirth(..)) && args(date, ..);

    after(Object date) : callSetBirth6(date) {
        System.out.println("6: user is born in " + date);
    }
    
    pointcut callSetBirth7(Object date) :
        call(public void User.setBirth(..)) && args(.., date);

    pointcut callSetBirth8(int month) :
        call(public void User.setBirth(..)) && args(*, month, ..);
    
    /////   args / args mit this oder target   /////
    
    pointcut callSetBirth9(Object date, User user) :
        call(public void User.setBirth(..))
            && args(date)
            && target(user);

    after(Object date, User user) : callSetBirth9(date, user) {
        System.out.println("9: " + user.getName() + " was born at " + date);
    }
    
    /////   Args und Autoboxing in AspectJ 5   /////
    
    pointcut callSetBirth10() :
        call(public void User.setBirth(Long));
    
    // ja, ich weiss, dass er nicht match't, aber ich will's nicht wissen
    // (@SuppressAjWarnings) wird spaeter noch behandelt
    @SuppressAjWarnings("adviceDidNotMatch")
    after() : callSetBirth10() {
        System.out.println("10: *** IT'S IMPOSSIBLE!!! *** ");
    }
    
    pointcut callSetBirth11() :
        call(public void User.setBirth(*)) && args(Long);
    
    after() : callSetBirth11() {
        System.out.println("11: args(Long) / AUTOBOXING");
    }
    
    
    /////   Kontrollfluss-basierte Bezeichner / cflow   /////

    pointcut setName2() :
        set(private String User.name)
            && cflow(call(public void setName(String)));

    after() : setName2() {
        System.out.println(thisJoinPoint + " from "
            + thisJoinPoint.getSourceLocation());
    }

    pointcut setName3(String name) :
        set(private String User.name)
            && !cflow(call(public void setName(String)))
            && args(name);

    after(String name) : setName3(name) {
        System.out.println("name set to " + name
                + " in " + thisJoinPoint.getSourceLocation());
    }

    /////   Kontrollfluss-basierte Bezeichner / cflowbelow   /////
    
    pointcut callSetBirthA() :
        call(public void setBirth(..));

    before() : callSetBirthA() {
        System.out.println("A: " + thisJoinPoint);
    }

    pointcut callSetBirthB() :
        call(public void setBirth(..))
            && !cflowbelow(call(public void setBirth(..)));

    before() : callSetBirthB() {
        System.out.println("B: " + thisJoinPoint);
    }

    pointcut callSetBirthC() :
        call(public void setBirth(..));

    before() : callSetBirthC() && !cflowbelow(callSetBirthC()) {
        System.out.println("C: " + thisJoinPoint);
    }

    before() : call(public void setBirth(..)) &&
            !cflowbelow(call(public void setBirth(..))) {
        System.out.println("-: " + thisJoinPoint);
    }

}
