/*
 * Created on Dec 7, 2003
 */
package bank;

import java.io.*;
import org.exolab.castor.xml.*;

/**
 * @author oliver
 */
public class Konto implements Persistence {
    
    protected int kontonummer;
    protected String inhaber;
    protected double kontostand;
    
    public Konto(int kontonr, String inhaber) {
        this.kontonummer = kontonr;
        this.inhaber     = inhaber;
        this.kontostand  = 0.0;
    }
    
    public Konto(int kontonr) {
        this(kontonr, null);
    }
    
    public double abfragen() {
        return kontostand;
    }
    
    public void einzahlen(double betrag) {
        kontostand = kontostand + betrag;
    }
    
    public void abheben(double betrag) {
        kontostand = kontostand - betrag;
    }
    
    public void ueberweisen(double betrag, Konto anderesKonto) {
        // hier gehoert eigentlich eine Transaktionsklammmer drumrum
        this.abheben(betrag);
        anderesKonto.einzahlen(betrag);
    }
    
    public String toString() {
        return "Konto(" + this.inhaber + "): " + this.kontostand;
    }
    
    public String getId() {
        return Integer.toString(this.kontonummer);
    }

    public boolean equals(Object other) {
        if (other instanceof Konto) {
            return this.equals((Konto) other);
        } else {
            return false;
        }
    }

    public boolean equals(Konto other) {
        return (this.kontonummer == other.kontonummer)
            && this.inhaber.equals(other.inhaber)
            && (this.kontostand == other.kontostand);
    }
        

        
    /**
     * public construcor is need for Castor to be able to read the XML
     */    
    public Konto() {}



    //  setter and getter are needed for Castor's marshalling and unmarshalling
        
    public void setKontonummer(int kontonr) {
        this.kontonummer = kontonr;
    }
        
    public void setInhaber(String name) {
        this.inhaber = name;
    }
        
    public void setKontostand(double value) {
        this.kontostand = value;
    }
        
    public int getKontonummer() {
        return this.kontonummer;
    }
        
    public String getInhaber() {
        return this.inhaber;
    }
        
    public double getKontostand() {
        return this.kontostand;
    }



    //  implementing the Persistence interface
    
    public Persistence load() throws IOException {
        Reader reader = new FileReader(getFilename());
        try {
            Unmarshaller unmarshaller = new Unmarshaller(Konto.class);
            return (Konto) unmarshaller.unmarshal(reader);
        } catch (CastorException e) {
            throw new IOException(e.getMessage());
        }
    }

    public void store() throws IOException {
        Writer writer = new FileWriter(getFilename());
        try {
            Marshaller.marshal(this, writer);
        } catch (CastorException e) {
            throw new IOException(e.getMessage());
        }
        writer.close();
    }
    
    public File getFilename() {
        return new File("Konto", this.getId());
    }

}
