package de.aosd;

/**
 * Dieses Beispiel stammt direkt aus dem Developers Notebook, um die Generics
 * mit M4 auszuprobieren.
 * 
 * @author oliver
 * @since 22.10.2005
 */
public abstract aspect ExceptionHandling<T extends Throwable> {

    /**
     * method to be implemented by sub-aspects to handle thrown exceptions
     */
    protected abstract void onException(T anException);
    
    /**
     * to be defined by sub-aspects to specify the scope of exception handling
     */
    protected abstract pointcut inExceptionHandlingScope();
    
    /**
     * soften T within the scope of the aspect
     */
    declare soft: T : inExceptionHandlingScope();
    
    /**
     * bind an exception thrown in scope and pass it to the handler
     */
    after() throwing (T anException) : inExceptionHandlingScope() {
        onException(anException);
    }

}
