/*
 * Created on 11.02.2005
 *
 * $Id: LogBook.aj,v 1.1 2005/06/24 13:56:05 boehm Exp $
 */
package space;

import java.lang.annotation.*;
import java.lang.reflect.AnnotatedElement;

import org.apache.log4j.Logger;
import org.aspectj.lang.*;
import org.aspectj.lang.reflect.*;

import space.annotation.*;
import space.annotation.archetype.Role;



/**
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 11.02.2005
 * @version $revision$
 */

@RegulatedBy(law="council regulation 2010R2D2")
public aspect LogBook {
    
    @Logging
    static Logger log = Logger.getLogger(LogBook.class);
    static Logger log1 = Logger.getLogger("LogBook.1");
    static Logger log2 = Logger.getLogger("LogBook.2");
    static Logger log3 = Logger.getLogger("LogBook.3");
    static Logger log4 = Logger.getLogger("LogBook.4");
    static Logger log5 = Logger.getLogger("LogBook.5");
    static Logger log6 = Logger.getLogger("LogBook.6");
    static Logger log7 = Logger.getLogger("LogBook.7");

    @PointOfInterest
    pointcut getIn() :
        execution(public void Taxi.enter(Passenger));
    
    @PointOfInterest
    pointcut getOut() :
        execution(public void Taxi.leave*(..));
    
    @Logging
    after(Passenger p) : getIn() && args(p) {
        log.info("passenger " + p + " entered");
    }
    
    @Logging
    after() : getOut() {
        log.info("passenger(s) lost");
    }
    
    
    
    //  ----  Annotation-Muster  ----
    
    pointcut queries() :
        execution(@Query * Taxi.*(..));
    
    after() returning(Object x): queries() {
        log1.debug("query = " + x + " for "+ thisJoinPoint.toShortString());
    }
    
    pointcut nonQueries() :
        execution(!@Query * Taxi.*(..));
    
    after() : nonQueries() {
        log1.debug("non query " + thisJoinPoint.toShortString());
    }
    
    pointcut dummyQueries() :
        execution(@Dummy @Query * Taxi.*(..));
        
    after() returning(Object x): dummyQueries() {
        log1.debug("dummy query = " + x + " for "+ thisJoinPoint.toShortString());
    }
    
    /**
     * Dieser Pointcut beinhaltet z.B. auch die getPassenger(int)-Methode
     */
    pointcut dummiesOrQueries() :
        execution(@(Dummy || Query) * Taxi.*(..));

    after() : dummiesOrQueries() {
        log.debug("*** dummy or query "+ thisJoinPoint.toShortString());
    }
    
    
    
    //  ----  Typ-Muster  ----
    
    /**
     * Das funktioniert leider noch nicht ganz, wie erwartet
     * (AspectJ version: 1.5.0M1):
     * 
     * ((@Alien *) *.new(..)) -> 
     * Syntax error on token "(", "constructor patterns have no return type" expected
     *
     * (@Alien *) Passenger+.new(..)) -> ok, but no result
     *
     * ((@Alien *) .new(..)) -> seems to be ok, test cases results in:
     * ... Klingon K. Lin Gon
     * ... Martian M. Ars
     */
    //after(Object x) : execution((@Alien *) *.new(..)) && this(x) {
    //after(Object x) : execution((@Alien *) Passenger+.new(..)) && this(x) {
    //after(Object x) : execution(@Alien Passenger+.new(..)) && this(x) {
    after(Object x) : execution((@Alien *).new(..)) && this(x) {
        log2.info("new ALIEN -> " + x);
    }
    
    after(Object x) : execution((!@Alien space.inhabitant.*).new(..)) && this(x) {
        log2.info("new NON-ALIEN -> " + x);
    }
    
    /**
     * Why does a Passenger class appears in the result set?
     * The Passenger class does not have a @Alien annotation. Also the
     * Earthling class appears. Is it a problem with paranthesis?
     * 
     * seems to be fixed with AspectJ 1.5.0M2
     */
    after(Object x) : execution((@Alien (space.* || space.inhabitant.*)).new(..)) 
            && !within(LogBook) && this(x) {
        log2.debug("(1) more ALIENs -> " + x);
    }
    
    /**
     * That 'or' combination works as expected
     */
    after(Object x) : execution((@Alien space.* || @Alien space.inhabitant.*).new(..)) 
            && !within(LogBook) && this(x) {
        log2.debug("(2) more ALIENs -> " + x);
    }
    
    /**
     * (@Alien Passenger+): includes classes Earthling and Passenger,
     *                      which don't have a @Alien annotation
     * (@Alien *)         : only Martian and Klingon class (which is ok)
     * 
     * seems to be fixed in AspectJ 1.5.0M2
     */
    after(Object x) : execution(((@Alien Passenger+) || space.Taxi).new(..)) 
            && !within(LogBook) && this(x) {
        log2.info("ALIEN or TAXI -> " + x);
    }
    
    after(Object x) : execution((@(Alien || Human) space.inhabitant.*).new(..))
            && !within(LogBook) && this(x) {
        log2.info("ALIEN or HUMAN -> " + x);
    }
    
    after(Object x) : execution((@Alien @Green space.inhabitant.*).new(..))
            && !within(LogBook) && this(x) {
        log2.info("new GREEN ALIEN -> " + x);
    }
    
    after(Object x) : execution((@(@Inherited *) space.inhabitant.*).new(..)) && this(x) {
        log2.info("new Inhabitant " + x);
    }
    
    
    
    //  ----  Signatur-Muster  ----
    
    /**
     * Beispiele fuer Felder
     */
    pointcut setConstantField() :
        //set (@Constant * *);
        //set (@Constant double space.Taxi.*);
        //set ((@Role *) space.Taxi.*);
        //set (@Human (@Role *) space.Taxi.*);
        //set (@Constant @Tariff * *);
        set (@Constant private * Taxi.*);
    
    after(Object value) : setConstantField() && args(value) {
        log3.info(thisJoinPoint.toShortString() + " = " + value);
    }

    /**
     * Beispiele fuer Methoden
     */
    pointcut Queries() :
        //call(@Query * *(..));
        //call(@Query * space.Taxi.get*(..));
        //call(@Query (@Role *) space.Taxi.get*(..));
        call(* *.*(@Role *));
    
    after() returning(Object x): Queries() && !within(LogBook) {
        log3.info(thisJoinPoint.toShortString() + " = " + x);
    }
    
    
    
    //  ----  Beispiel-Pointcuts  ----
    
    pointcut setConstant() :
        set (@Constant * *);
    
    after(Object value) : setConstant() && args(value) && !within(LogBook) {
        log4.info("Constant " + thisJoinPoint.toShortString() + " = " + value);
    }

    pointcut callCriticals() :
        call(* (@Critical *).*(..)) || call(@Critical * *(..));
    
    /*
     * FareCalculator ist als @Critical vermerkt, sollte aber nicht im
     * Log auftauchen, da es auch als @Secret gekennzeichnet ist.
     */
    after() returning(Object x): (Queries() || callCriticals())
            && !within(@Secret *) {
        log4.info("critical or queries: " + thisJoinPoint.toShortString()
                + " = " + x);
    }
    
    before() : handler(@Critical *) {
        log.info("exception handled in " + thisJoinPoint.getSourceLocation());
    }
    
    
    
    //  ====  Annotations zur Laufzeit  ====
    
    //  ----  @this, @target und @args  ----
    
    after() : call(public * *.*(..)) && @target(Role) && !within(LogBook)
            //&& !cflowbelow(execution(String *.toString())) {
            && !withincode(String *.toString()) {
        log5.info(thisJoinPoint.toShortString() + " from " 
                + thisJoinPoint.getSourceLocation());
    }

    before() : call(public * *.*(..)) && @this(Role) 
            && !withincode(String *.toString())
            //&& !cflowbelow(execution(String *.toString()))
            && !within(LogBook) {
        log5.info("@Role " + thisJoinPoint.getThis() + ": "
                + thisJoinPoint.toShortString());
    }

    before(Licence licence) : execution(*.new(..)) && @target(licence) {
        log5.info("instantiating licensed class");
    }

    before(Licence licence) : execution(*.new(..)) && @target(licence)
              && if(licence.value().equals("GPL")) {
        log5.info("GPL: instantiating GPL'd "
                + thisJoinPoint.getTarget().getClass());
    }

    /**
     * This example results in a
     * java.lang.VerifyError: (class: space/Taxi, method: leave signature: (I)V) Expecting to find object/array on stack
     *     at space.test.TaxiTest.setUp(TaxiTest.java:30)
     *     at junit.framework.TestCase.runBare(TestCase.java:125)
     *     ...
     * 
     * @see https://bugs.eclipse.org/bugs/show_bug.cgi?id=87656
     */
//    after() : execution(public * Taxi.*(..)) && @args(Role) {
//        log5.info("@Role args: " + thisJoinPoint.toShortString());
//    }
    
    
    
    //  ----  @within, @withincode und @annotation  ----
    
    after() : call(* space..*(..)) && @within(Test) {
        log6.debug("@within(Test): " + thisJoinPoint);
    }

    after() : call(* *.*(..)) && @withincode(Critical) {
        log6.debug("@withincode(Critical): " + thisJoinPoint.toShortString()
                + " from " + thisJoinPoint.getSourceLocation());
    }

    after() : call(* *.*(..)) && @annotation(Critical) {
        log6.info("@annotation(Critical): " + thisJoinPoint);
    }
    
    //  alternativ:
    after() : call(@Critical * *.*(..)) {
        log6.info("********************** " + thisJoinPoint);
    }
    
    
    
    //  ====  Auf Annotations zugreifen  ====
    
    //  ----  Das Beförderungsgesetz  ----

    before(Taxi taxi, Passenger x) throws PassengerException : 
            execution(public void Taxi.enter(Passenger)) 
            && this(taxi) && args(x) {
        Annotation a = getAnnotation(x);
        if (a == null) {
            throw new PassengerException(x + " has no annotation");
        }
        for (int i = 1; i <= 4; i++) {
            Passenger p = taxi.getPassenger(i);
            if (p != null) {
                if (!a.equals(getAnnotation(p))) {
                    log7.error("security check failed for " + x);
                    throw new PassengerException("don't mix humans and aliens");
                }
            }
        }
    }

    private Annotation getAnnotation(Passenger p) {
        Class<? extends Passenger> cl = p.getClass();
        Annotation a = cl.getAnnotation(Human.class);
        if (a == null) {
            a = cl.getAnnotation(Alien.class);
        }
        return a;
    }


    
    //  ----  MemberSignature.getAccessibleObject()  ----
    
    after() : call(@Query * *(..)) {
        logAnnotations(thisJoinPoint);
    }
    
    private void logAnnotations(JoinPoint joinpoint) {
        Signature sig = joinpoint.getSignature();
        log7.info("@Query: calling " + sig);
        if (sig instanceof MemberSignature) {
            AnnotatedElement annotationInfo = getAnnotatedElement((MemberSignature) sig);
            log7.info("annotationInfo = " + annotationInfo);
        }
    }
    
    /*
     * MemberSignature.getAccessibleObject() geht leider noch nicht
     * (Stand: 07-Mar-05)
     */
    private AnnotatedElement getAnnotatedElement(MemberSignature sig) {
        //return sig.getAccessibleObject();
        return null;
    }

}

