/*
 * Created on Nov 25, 2003
 */
package bio;

import java.io.*;
import java.text.*;
import java.util.Date;

/**
 * Rhythm berechnet den persoenlichen Biorythums nach folgenden Regeln:
 * <ul>
 *   <li> geistig (mental)       = sin (days * 2 * pi / 23) </li>
 *   <li> koerperlich (physical) = sin (days * 2 * pi / 28) </li>
 *   <li> seelisch (emotional)   = sin (days * 2 * pi / 33) </li>
 * </ul>
 * @author oliver
 * @since  Nov 25, 2003
 */
public class Rhythm {
    
    private Date birth = null;

    public Rhythm(String birth) throws ParseException {
        this(new SimpleDateFormat("d.M.y").parse(birth));
    }

    public Rhythm(Date birth) {
        this.birth = birth;
    }
    
    public double getAgeInDays() {
        return (new Date().getTime() - birth.getTime()) / 86400000.0;
    }
    
    static private int inPercent(double x) {
        return (int) Math.round(x * 100);
    }
    
    /**
     * seelisch (emotional) = sin (days * 2 * pi / 33) 
     * @return between +100% and -100%
     */
    public int getEmotional() {
        return inPercent(Math.sin(this.getAgeInDays() * 2 * Math.PI / 33));
    }
    
    /**
     * geistig (mental) = sin (days * 2 * pi / 23) 
     * @return between +100% and -100%
     */
    public int getMental() {
        return inPercent(Math.sin(this.getAgeInDays() * 2 * Math.PI / 23));
    }
    
    /**
     * koerperlich (physical) = sin (days * 2 * pi / 28) 
     * @return between +100% and -100%
     */
    public int getPhysical() {
        return inPercent(Math.sin(this.getAgeInDays() * 2 * Math.PI / 28));
    }
    
    public boolean equals(Object other) {
        if (other instanceof Rhythm) {
            return this.equals((Rhythm) other);
        } else {
            return false;
        }
    }

    public boolean equals(Rhythm other) {
        return birth.equals(other.birth);
    }
    
    public int hashCode() {
        return birth.hashCode();
    }
    
    public String toString() {
        return "Alter: " + getAgeInDays() + " Tage";
    }
    
//    private static String readName() {
//        System.out.print("Name: ");
//        String name = "";
//        BufferedReader reader =
//            new BufferedReader(new InputStreamReader(System.in));
//        name = reader.readLine();
//        return name;
//    }
    
    public static void main(String[] args) throws Exception {
        //readName();
        System.out.print("Geburtsdatum: ");
        String birth = null;
        if (args.length == 0) {
            BufferedReader reader =
                new BufferedReader(new InputStreamReader(System.in));
            birth = reader.readLine();
        } else {
            birth = args[0];
            System.out.println(birth);
        }
        Rhythm rhythm = new Rhythm(birth);
        System.out.println("Ihr Biorhythmus:");
        System.out.println("\tgeistig:     " + rhythm.getMental() + "%");
        System.out.println("\tseelisch:    " + rhythm.getEmotional() + "%");
        System.out.println("\tkoerperlich: " + rhythm.getPhysical() + "%");
    }
}
