/*
 * Created on 23.05.2005
 *
 * $Id: Robot2.java,v 1.1 2005/06/24 13:56:06 boehm Exp $
 */
package web;

import java.io.IOException;
import java.net.*;
import java.net.Socket;


/**
 * Ein einfacher Robot, der ausgeschickt wird, um das Web zu erforschen.
 * Diese Implementierung verwendet den SocketPool, um den teuren
 * Verbindungsaufbau zum minimieren.
 * 
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 26.05.2005
 * @version $revision$
 */
public class Robot2 {
    
    public String hostname = null;
    
    private SocketPool socketPool = new SocketPool(80);
    
    public Robot2() {
        this("localhost");
    }
    
    public Robot2(String hostname) {
        this.hostname = hostname;
    }
    
    /**
     * Ein einfacher Verbindungsaufbau, um festzustellen, ob der
     * gewuenschte Webserver erreichbar ist.
     * 
     * @return Antwort des Webservers bzw. Fehlermeldung
     */
    public String ping() {
        try {
            Socket socket = socketPool.getSocket(hostname);
            if (socket != null) {
                // hier jetzt auf den Socket zugreifen
                // und die Antwort des Servers interpretieren
                // ...
                socketPool.putSocket(socket);
                return this.hostname + ":80 is alive";
            }
        } catch (UnknownHostException e) {
            return "unknown host " + hostname;
        } catch (IOException e) {
            return this.hostname + ":80: " + e;
        }
        return "no answer from " + this.hostname;
    }
    
    public String ping(int n) {
        String answer = null;
        long t0 = System.currentTimeMillis();
        for (int i = 0; i < n; i++) {
            answer = ping();
        }
        long t1 = System.currentTimeMillis();
        return n + " pings in " + (t1 -t0) + " ms - " + answer;
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        Robot2 robot = new Robot2();
        System.out.println(robot.ping());
        System.out.println(robot.ping(3));
    }

}
