/*
 * Created 20-Nov-04
 */
package toolbox;

/**
 * @author mailto:boehm@javatux.de
 */
public aspect ProfilerAspect {
    
    private LogTable logTable = new LogHashTable();
    
    pointcut methodsToMeasure() : 
        (execution(* *..*(..))
        || execution(*..new(..)))
        && !within(*..*Test)
        && !within(toolbox.*);
    
    /**
     * ermittelt Zeit-verbrauch
     */
    Object around() : methodsToMeasure() {
        long t = System.currentTimeMillis();
        long mem = Runtime.getRuntime().freeMemory();
        try {
            return proceed();
        } finally {
            t = System.currentTimeMillis() - t;
            mem = mem - Runtime.getRuntime().freeMemory();
            logTable.add(thisJoinPoint, t, mem);
        }
    }
    
    after() : execution(public static void *..*.main(..)) {
        logTable.exit();
    }

}
