/*
 * Created on 02.04.2005
 *
 * $Id: Config.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 */
package armada;

import java.io.*;
import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.Logger;

/**
 * It's implemented as singleton to avoid a initialization error because of
 * problems reading the property file.
 *  
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 02.04.2005
 * @version $revision$
 */
public class Config {
    
    /** where to log */
    private static Logger log = Logger.getLogger(Config.class);
    
    /** the only instance of this class */
    static private Config instance = null;
    
    private Properties properties = null;
    
    private Config() {
        properties = new Properties();
        loadProperties();
    }
    
    static public Config getInstance() {
        if (instance == null) {
            instance = new Config();
        }
        return instance;
    }

    private void loadProperties() {
        loadProperties(Config.class.getResourceAsStream("/armada.properties"));
        log.info("properties loaded");
    }
    
    private void loadProperties(InputStream istream) {
        try {
            properties.load(istream);
        } catch (IOException e) {
            System.err.println("can't load armada.properties");
        }
    }
    
    public String getProperty(String key) {
        String value = properties.getProperty(key);
        log.debug("property " +  key + " = " + value);
        return value;
    }
    
    public int getBoardWidth() {
        int width;
        try {
            width = Integer.parseInt(getProperty("board.width"));
        } catch (NumberFormatException e) {
            width = 10;
        }
        log.debug("property board.width = " + width);
        return width;
    }

    public int getBoardHeight() {
        int height;
        try {
            height = Integer.parseInt(getProperty("board.height"));
        } catch (NumberFormatException e) {
            height = 10;
        }
        log.debug("property board.height = " + height);
        return height;
    }

}
