/*
 * $Id: BoardView.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 *
 * Copyright (c) 2001 Oliver Boehm (mailto:boehm@2xp.de)
 * http://www.2xp.de
 *
 * -----------------------------------------
 *
 * $Author: boehm $
 * $Date: 2005/06/24 13:56:04 $
 * $Revision: 1.1 $
 *
 * $Log: BoardView.java,v $
 * Revision 1.1  2005/06/24 13:56:04  boehm
 * Einspielen der Beispiele und Uebungen inkl. HTML-Seiten
 *
 * Revision 1.13  2002/04/18 13:17:35  oliver
 * minor corrections
 *
 * Revision 1.12  2001/12/27 16:47:04  oliver
 * in BoardView the ships are only shown on the right side
 *
 * Revision 1.11  2001/12/27 16:21:33  oliver
 * HumanPlayer and Player.main() added
 *
 * Revision 1.10  2001/12/26 09:50:18  oliver
 * computer plays against computer
 *
 * Revision 1.9  2001/12/23 22:54:46  oliver
 * main() added to Player and tested intensiv
 *
 * Revision 1.8  2001/11/10 21:04:43  oliver
 * test classes added
 *
 * Revision 1.7  2001/10/15 20:24:42  oliver
 * merge with stans_bugfix_branch
 *
 * Revision 1.4.2.2  2001/10/14 19:29:23  stan
 * show() renamed to print()
 *
 * Revision 1.4.2.1  2001/10/14 19:04:16  stan
 * show() changed
 *
 * Revision 1.4  2001/10/09 21:04:24  oliver
 * main added()
 *
 * Revision 1.3  2001/10/09 20:00:05  stan
 * comment changed
 *
 * Revision 1.2  2001/10/09 19:50:29  stan
 * first prototype of show()
 *
 * Revision 1.1  2001/10/07 10:25:15  oliver
 * BoardView.java added
 *
 */



package armada;



import java.io.*;



/**
 * Title:        Armada
 * Description:  Ein einfaches "Schiffe versenken"-Beispiel
 * Copyright:    Copyright (c) 2001
 * @author  Oliver Boehm (boehm@javatux.de)
 * @version 1.0
 */

public class BoardView {

    /** the boards which should be viewed (the model in the MVC pattern) */
    private Board board1;
    private Board board2;



    /**
     * Constructor BoardView
     * @post (board != null)
     */

    public BoardView() {
        board1 = new Board();
        board2 = new Board();
    }
    
    public BoardView(Board board) {
    	this.board1 = board;
    	this.board2 = new Board();
    }
    
    public BoardView(Board board1, Board board2) {
    	this.board1 = board1;
    	this.board2 = board2;
    }



    /**
     * show the board on the screen
     */
    public void print() {
        StringWriter writer = new StringWriter();
        try {
            this.print(writer);
            System.out.print(writer);
        } catch (IOException e) {
            System.err.println("can't show board!\n" + e);
        }
    }

    /**
     * prints the boards into the given writer.
     * The left side is printed with the shots and hits only,
     * the right side is shown with the set ships.
     *
     * @params  writer  where to write the board
     */
    public void print(Writer writer) throws IOException {
        writer.write("\n");
        printBoarder(writer);
        for (int y = 0; y < board1.height; y++) {
            printHitsOnly(writer, y, board1);
            writer.write("          ");
            printHitsAndShips(writer, y, board2);
            writer.write("\n");
        }
        printBoarder(writer);
        writer.write("\n");
        printShipItems(writer, board1);
        writer.write("       ");
        printShipItems(writer, board2);
        writer.write("\n\n");
    }

    /**
     * Prints the horizontal boarder ("  -A-B-C-...");
     * @param writer
     * @throws IOException
     */
    private void printBoarder(Writer writer) throws IOException {
        writer.write("       ");
        printBoarderMarking(writer);
        writer.write("            ");
        writer.write("       ");
        printBoarderMarking(writer);
        writer.write("\n");
    }
    
    /**
     * Printer the boarder marking "-A-B-C-..."
     * @param writer
     * @throws IOException
     */
    private void printBoarderMarking(Writer writer) throws IOException {
        writer.write("-");
        for (char x = 'A'; x < 'A' + board1.width; x++) {
            writer.write(x);
            writer.write("-");
        }
    }
    
    private void printHitsAndShips(Writer writer, int y, Board board)
    		   					throws IOException {
        writer.write("     " + y + "|");
        for (char x = 'A'; x < 'A' + board.width; x++) {
            char hit = board.getField(x, y);
            switch (hit) {
            	case Board.NOTHING:
            	case Board.EMPTY:	writer.write("  ");
            				break;
            	case Board.HIT:		writer.write(" #");
            				break;
            	case Board.NO_HIT:	writer.write(" ~");
            				break;
            	default:		writer.write(" " + hit);
            }
        }
        writer.write(" |" + y);
    }
    
    private void printHitsOnly(Writer writer, int y, Board board)
    		   					throws IOException {
        writer.write("     " + y + "|");
        for (char x = 'A'; x < 'A' + board.width; x++) {
            char hit = board.getField(x, y);
            switch (hit) {
            	case Board.NOTHING:
            	case Board.EMPTY:	writer.write("  ");
            				        break;
            	case Board.HIT:		writer.write(" #");
            				        break;
            	case Board.NO_HIT:	writer.write(" ~");
            				        break;
            	default:		    writer.write("  ");
            }
        }
        writer.write(" |" + y);
    }
    
    private void printShipItems(Writer writer, Board board) 
    				   			throws IOException {
    	writer.write(" ");
        for (char ship = '4'; ship >= '1'; ship--) {
             writer.write(" " + ship + "er:"
             		  + board.getShipHits(ship) + "/"
             		  + board.getShipItems(ship));	
        }
    	
    }
    
    public void printShot(Shot shot) {
        StringWriter writer = new StringWriter();
        try {
            this.printShot(writer, shot);
            System.out.print(writer);
        } catch (IOException e) {
            System.err.println("can't show shot!\n" + e);
        }
    }
    
    public void printShot(Writer writer, Shot shot) throws IOException {
    	printShot1(writer, shot);
    	writer.write("\n\n");
    }
    
    /**
     * prints the shot on the left side of the screen.
     */
    public void printShot1(Writer writer, Shot shot) throws IOException {
    	char x = 'A';
    	x += (shot.x - 1);
    	int  y = shot.y - 1;
    	writer.write("     Schuss: " + x + y);
    	if (shot.isHit()) {
    	    writer.write(" " + shot.hit + "er ");
    	    if (shot.sunkShip()) {
    	    	writer.write("versenkt");
    	    } else {
    	    	writer.write("Treffer ");
    	    }
    	} else {
    	    writer.write("             ");
    	}
    	writer.write("            ");
    }
    

   
    public void printShot2(Shot shot) {
        StringWriter writer = new StringWriter();
        try {
            this.printShot2(writer, shot);
            System.out.print(writer);
        } catch (IOException e) {
            System.err.println("can't show shot!\n" + e);
        }
    }
    
    
    
    public void printShot2(Writer writer, Shot shot) throws IOException {
    	writer.write("                                        ");
    	printShot(writer, shot);
    }
    
    
    
    /**
     *  prints both shots in one line.
     */
    
    public void printShots(Writer writer, Shot shot1, Shot shot2) 
    				throws IOException{
    	this.printShot1(writer, shot1);
    	this.printShot1(writer, shot2);
    	writer.write("\n\n");
    }
    
    public void printShots(Shot shot1, Shot shot2){
        StringWriter writer = new StringWriter();
        try {
    	    this.printShots(writer, shot1, shot2);
            System.out.print(writer);
        } catch (IOException e) {
            System.err.println("can't show shots!\n" + e);
        }
    }
    




    /**
     * main method (for testing)
     * @param args
     */
    public static void main(String[] args) {
        BoardView view = new BoardView();
        view.print();
    }
}
