/*
 * Created on 02.04.2005
 *
 * $Id: Config.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 */
package armada;

import java.io.*;
import java.util.Properties;

/**
 * It's implemented as singleton to avoid a initialization error because of
 * problems reading the property file.
 *  
 * @author <a href="mailto:boehm@javatux.de">oliver</a>
 * @since 02.04.2005
 * @version $revision$
 */
public class Config {
    
    /** the only instance of this class */
    static private Config instance = null;
    
    private Properties properties = null;
    
    private Config() {
        properties = new Properties();
        loadProperties();
    }
    
    static public Config getInstance() {
        if (instance == null) {
            instance = new Config();
        }
        return instance;
    }

    private void loadProperties() {
        loadProperties(Config.class.getResourceAsStream("/armada.properties"));
    }
    
    private void loadProperties(InputStream istream) {
        try {
            properties.load(istream);
        } catch (IOException e) {
            System.err.println("can't load armada.properties");
        }
    }
    
    public String getProperty(String key) {
        return properties.getProperty(key);
    }
    
    public int getBoardWidth() {
        try {
            return Integer.parseInt(getProperty("board.width"));
        } catch (NumberFormatException e) {
            return 10;
        }
    }

    public int getBoardHeight() {
        try {
            return Integer.parseInt(getProperty("board.height"));
        } catch (NumberFormatException e) {
            return 10;
        }
    }

}
