/*
 * Created on 03.04.2005
 *
 * $Id: LogAspect.aj,v 1.1 2005/06/24 13:56:04 boehm Exp $
 */
package armada;

import org.apache.log4j.Logger;
import org.aspectj.lang.*;

public aspect LogAspect {

    private static Logger log = Logger.getLogger(LogAspect.class);

    before() : execution(Board.new(int, int))
            || execution(private void Board.markWithShip(int, int, int, int, int))
            || execution(Player+.new(String)) {
        log.info(getAsString(thisJoinPoint));
    }
    
    after() : execution(private void Config.loadProperties()) {
        log.info(getAsString(thisJoinPoint) + " finished");
    }
    
    after() returning (Object ret):
            execution(private boolean Board.shipCollidet(int, int, int ,int))
            || execution(private int Board.getShipIntactItems(char))
            || execution(public Shot Player+.turn())
            || execution(private Shot ComputerPlayer.guessNextHit())
            || execution(public * Config.get*()) {
        log.info(getAsString(thisJoinPoint) + " = " + ret);
    }
    
    after(Shot x) : execution(Shot.new(..)) && this(x) {
        log.info("new Shot " + x);
    }
    
    after(String msg) : execution(ShotException.new(String)) && args(msg) {
        log.warn("new ShotExcption(" + msg +")");
    }

    
    
    private static String getAsString(JoinPoint joinpoint) {
        Signature sig = joinpoint.getSignature();
        return sig.getDeclaringType().getName() + "."
                + sig.getName() + "(" + getArgsAsString(joinpoint.getArgs())
                + ")";
    }
    
    private static String getArgsAsString(Object[] args) {
        if ((args == null) || (args.length == 0)) {
            return "";
        }
        String s = args[0].toString();
        for (int i = 1; i < args.length; i++) {
            s += ", " + args[i].toString();
        }
        return s;
    }
    
}
