/*
 * Created on 03.04.2005
 *
 * $Id: PersistenceAspect.aj,v 1.1 2005/06/24 13:56:04 boehm Exp $
 */
package armada;

import java.io.*;

public aspect PersistenceAspect {

    public void Player.store() throws IOException {
        OutputStream ostream = new FileOutputStream("board." + name);
        store(new ObjectOutputStream(ostream));
        ostream.close();
    }
    
    /**
     * The opponent is set to null to avoid recursion during writing the
     * different attributes.
     * 
     * @param oostream
     * @throws IOException
     */
    private void Player.store(ObjectOutputStream oostream) throws IOException {
        Player opponent = this.opponent;
        this.opponent = null;
        oostream.writeObject(this);
        this.opponent = opponent;
    }
    
    public Player Player.load() throws IOException {
        InputStream istream = new FileInputStream("board." + name);
        Player player = load(new ObjectInputStream(istream));
        istream.close();
        return player;
    }
    
    private Player Player.load(ObjectInputStream oistream) throws IOException {
        try {
            return (Player) oistream.readObject();
        } catch (ClassNotFoundException e) {
            throw new IOException("unexpected " + e);
        }
    }
    
    public void Player.unload() {
        File file = new File("board." + name);
        file.delete();
    }
    
    
    
    /**
     * Store automatically after each turn.
     */
    after(Player player) : execution(public void Player+.turn())
                           && this(player) {
        try {
            player.store();
        } catch (IOException e) {
            System.err.println("can't store score");
        }
    }
    
    void around(Player player1, Player player2) :
            execution(public static void play(Player, Player))
            && args(player1, player2) {
        try {
            player1 = player1.load();
            player2 = player2.load();
            System.out.println("OLD GAME restored");
        } catch (IOException e) {
            System.out.println("beginning NEW GAME\n");
        }
        proceed(player1, player2);
        player1.unload();
        player2.unload();
    }

}
