/*
 * $Id: Player.java,v 1.1 2005/06/24 13:56:04 boehm Exp $
 *
 * Copyright (c) 2001 Oliver Boehm (mailto:boehm@2xp.de)
 * http://www.2xp.de
 * All rights reserved.
 *
 * -----------------------------------------
 *
 * $Author: boehm $
 * $Date: 2005/06/24 13:56:04 $
 * $Revision: 1.1 $
 *
 * $Log: Player.java,v $
 * Revision 1.1  2005/06/24 13:56:04  boehm
 * Einspielen der Beispiele und Uebungen inkl. HTML-Seiten
 *
 * Revision 1.9  2002/04/18 16:40:24  oliver
 * conflict merged
 *
 * Revision 1.8  2002/04/18 16:33:07  stan
 * javadoc comment for gameOver() inserted
 *
 * Revision 1.7  2001/12/27 16:21:33  oliver
 * HumanPlayer and Player.main() added
 *
 * Revision 1.6  2001/12/26 10:43:42  oliver
 * class ComputerPlayer added and made Player abstarct
 *
 * Revision 1.5  2001/12/26 09:50:18  oliver
 * computer plays against computer
 *
 * Revision 1.4  2001/12/23 22:54:46  oliver
 * main() added to Player and tested intensiv
 *
 * Revision 1.2  2001/12/22 20:52:23  oliver
 * AllTests added
 *
 * Revision 1.1.1.1  2001/10/06 15:23:10  oliver
 * imported sources
 */



package armada;

import java.io.*;






/**
 * Title:        Armada
 * Description:  Ein einfaches "Schiffe versenken"-Beispiel
 * Copyright:    Copyright (c) 2001
 * Company:      2XP
 * @author Oliver Boehm
 * @version 1.0
 */

abstract public class Player implements Serializable {

    private static final long serialVersionUID = 1L;

    /** the other player */
    public Player opponent = null;

    /** the player's board */
    public Board board = new Board();
    
    /** the player's name */
    public String name = null;

    
    
    public Player() {
        this("anonymous");
    }
    
    public Player(String name) {
        this.name = name;
    }



    /**
     * it's the players turn now
     */

    public abstract Shot turn();



    /**
     * Is the game over or not?
     *
     * @return  true (game over) or false (game not over)
     */

    public boolean gameOver() {
    	if ((board.getShipIntactItems() == 0) ||
    		(opponent.board.getShipIntactItems() == 0)) {
    	    return true;
    	} else {
    	    return false;
    	}
    }
    
    
    
    public boolean hasWon() {
    	return (opponent.board.getShipIntactItems() == 0);
    }
    
    
    
    public String toString() {
        return name;
    }
    
    
    
    public boolean equals(Object other) {
        if (other instanceof Player) {
            return equals((Player) other);
        } else {
            return false;
        }
    }

    public boolean equals(Player other) {
        return this.name.equals(other.name) && this.board.equals(other.board);
    }


    public static void main(String[] args) {
    	play(new HumanPlayer(), new ComputerPlayer());
    }

    public static void play(Player player1, Player player2) {
        player1.opponent = player2;
        player2.opponent = player1;
        BoardView view = new BoardView (player2.board, player1.board);
        view.print();
        while (!player1.gameOver()) {
    	    Shot shot1 = player1.turn();
    	    if (player2.gameOver()) {
    	    	break;
    	    }
     	    Shot shot2 = player2.turn();
    	    view.print();
    	    view.printShots(shot1, shot2);
    	}
        if (player1.hasWon()) {
    	    System.out.println("\n" + player1 + " hat gewonnen!\n");
    	} else {
    	    System.out.println("\n" + player2 + " hat gewonnen\n");
    	}
    }

}
